/*---------------------------------------------------------------------------*
 *                                   IT++			             *
 *---------------------------------------------------------------------------*
 * Copyright (c) 1995-2003 by Tony Ottosson, Thomas Eriksson, Pl Frenger,   *
 * Tobias Ringstrm, and Jonas Samuelsson.                                   *
 *                                                                           *
 * Permission to use, copy, modify, and distribute this software and its     *
 * documentation under the terms of the GNU General Public License is hereby *
 * granted. No representations are made about the suitability of this        *
 * software for any purpose. It is provided "as is" without expressed or     *
 * implied warranty. See the GNU General Public License for more details.    *
 *---------------------------------------------------------------------------*/

/*! 
  \file 
  \brief Implementation of Filter classes and functions.
  \author Hkan Eriksson and Thomas Eriksson

  1.9

  2003/05/22 08:55:19
*/

#include "base/filter.h"

namespace itpp {

  vec fir1(long N, double cutoff)
  {
    vec a(N+1),h=hamming(N+1);

    for (long i=0;i<length(a);i++) {
      a[i]=h[i]*sinc(cutoff*(i-N/2.0));
    }
    a/=sum(a);
    return a;
  }

  //-----------------------------------------------------------------------
  //  class Filter
  //-----------------------------------------------------------------------

  //! Template instantiation of Filter
  template Filter<double,double,double>;
  //! Template instantiation of Filter
  template Filter<double,complex<double>,complex<double> >;
  //! Template instantiation of Filter
  template Filter<complex<double>,double,complex<double> >;
  //! Template instantiation of Filter
  template Filter<complex<double>,complex<double>,complex<double> >;

  //-----------------------------------------------------------------------
  //  class MA_Filter
  //-----------------------------------------------------------------------

  //! Template instatiation of MA_Filter
  template MA_Filter<double,double,double>;
  //! Template instatiation of MA_Filter
  template MA_Filter<double,complex<double>,complex<double> >;
  //! Template instatiation of MA_Filter
  template MA_Filter<complex<double>,double,complex<double> >;
  //! Template instatiation of MA_Filter
  template MA_Filter<complex<double>,complex<double>,complex<double> >;

  //-----------------------------------------------------------------------
  //  class AR_Filter
  //-----------------------------------------------------------------------

  //! Template instatiation of AR_Filter
  template AR_Filter<double,double,double>;
  //! Template instatiation of AR_Filter
  template AR_Filter<double,complex<double>,complex<double> >;
  //! Template instatiation of AR_Filter
  template AR_Filter<complex<double>,double,complex<double> >;
  //! Template instatiation of AR_Filter
  template AR_Filter<complex<double>,complex<double>,complex<double> >;

  //-----------------------------------------------------------------------
  //  filter functions
  //-----------------------------------------------------------------------

  //! Template instantiation filter
  template vec filter<double,double,double>(const vec &b, const vec &a, const vec &input);
  //! Template instantiation filter
  template vec filter<double,double,double>(const vec &b, int one, const vec &input);
  //! Template instantiation filter
  template vec filter<double,double,double>(int one, const vec &a, const vec &input);

  //! Template instantiation filter
  template cvec filter<complex<double>,double,complex<double> >(const vec &b, const vec &a, const cvec &input);
  //! Template instantiation filter
  template cvec filter<complex<double>,double,complex<double> >(const vec &b, int one, const cvec &input);
  //! Template instantiation filter
  template cvec filter<complex<double>,double,complex<double> >(int one, const vec &a, const cvec &input);

  //! Template instantiation filter
  template cvec filter<complex<double>,complex<double>,complex<double> >(const cvec &b, const cvec &a, const cvec &input);
  //! Template instantiation filter
  template cvec filter<complex<double>,complex<double>,complex<double> >(const cvec &b, int one, const cvec &input);
  //! Template instantiation filter
  template cvec filter<complex<double>,complex<double>,complex<double> >(int one, const cvec &a, const cvec &input);

  //! Template instantiation filter
  template cvec filter<double,complex<double>,complex<double> >(const cvec &b, const cvec &a, const vec &input);
  //! Template instantiation filter
  template cvec filter<double,complex<double>,complex<double> >(const cvec &b, int one, const vec &input);
  //! Template instantiation filter
  template cvec filter<double,complex<double>,complex<double> >(int one, const cvec &a, const vec &input);

} //namespace itpp
